import {addLegend, addSvg, width, height as _height, margin, colors, loadData, getDimensions} from "./lib.js";

const height = _height-105

loadData().then(([schengenData1, countryData]) => {
    // Map country data by country name for quick lookup
    const countryMap = {};
    countryData.forEach(d => {
        countryMap[d.Country] = {
            gdp: +d.GDP.replace(/[$,]/g, ''),
            population: +d.Population.replace(/[$,]/g, '')
        };
    });

    // Merge GDP and population data into Schengen dataset
    schengenData1.forEach(d => {
        const countryInfo = countryMap[d["Schengen State"]];
        d.GDP = countryInfo ? countryInfo.gdp : 0;
        d.Population = countryInfo ? countryInfo.population : 0;
    });

    let sizeDim;
    let opacityDim;

    const svg = addSvg("#bubble-container", 0, 115)

    function updateBubble() {
        console.log("dims", sizeDim, opacityDim)
        const schengenData = Array.from(d3.rollup(
            schengenData1,
            v => ({
                GDP: v[0].GDP,
                Population: v[0].Population,
                "size": d3.sum(v, d => d[sizeDim]),
                "opacity": d3.sum(v, d => d[opacityDim])
            }),
            d => d["Schengen State"]
        )).map(([key, value]) => ({
            "Schengen State": key,
            GDP: value.GDP,
            Population: value.Population,
            "size": value["size"],
            "opacity": value["opacity"]
        }));

        console.log(schengenData)

        const xScale = d3.scaleLog()
            .domain([d3.min(schengenData, d => d.GDP), d3.max(schengenData, d => d.GDP)])
            .range([0, width-margin.left-margin.right]);

        const yScale = d3.scaleLog()
            .domain([d3.min(schengenData, d => d.Population), d3.max(schengenData, d => d.Population)])
            .range([height-margin.top-margin.bottom, 0]);

        const sizeScale = d3.scaleSqrt()
            .domain([0, d3.max(schengenData, d => d["size"] || 1)])
            .range([8, 60]);

        const opacityScale = d3.scaleSqrt()
            .domain([0, d3.max(schengenData, d => d["opacity"] || 1)])
            .range([0.1, 1]);

        const colorScale = d3
            .scaleOrdinal()
            .domain(schengenData)
            .range(colors)

        svg.selectAll("*").remove();

        svg.append("g")
            .attr("transform", `translate(0,${height-margin.top-margin.bottom})`)
            .call(d3.axisBottom(xScale).ticks(10, ".1s"))
            .append("text")
            .attr("class", "axis-label")
            .attr("transform", `translate(${(width-margin.left-margin.right)/2},50)`)
            .text("GDP")
            .style("fill", "black");

        svg.append("g")
            .call(d3.axisLeft(yScale))
            .append("text")
            .attr("class", "axis-label")
            //.attr("x", -height / 2)
            .attr("y", -50)
            .attr("transform", `rotate(-90) translate(${-(height-margin.top-margin.bottom)/2},-10)`)
            .text("Population")
            .style("fill", "black");

        const tooltip = d3.select("body")
            .append("div")
            .attr("class", "tooltip")
            .style("visibility", "hidden");

        svg.selectAll("circle")
            .data(schengenData)
            .enter()
            .append("circle")
            .attr("class", "circle")
            .attr("cx", d => xScale(d.GDP || 1e10))
            .attr("cy", d => yScale(d.Population || 1))
            .attr("r", d => Math.round(sizeScale(d["size"] || 1)))
            .style("opacity", d => opacityScale(d["opacity"] || 1))
            .attr("fill", d => colorScale(d["Schengen State"]))
            .on("mouseover", (event, d) => {
                tooltip.style("visibility", "visible")
                    .html(`
                            <strong>${d["Schengen State"]}</strong><br>
                            GDP: $${d.GDP.toLocaleString()}<br>
                            Population: ${d.Population.toLocaleString()}<br>
                            ${sizeDim}: ${d["size"].toLocaleString()}<br>
                            ${opacityDim}: ${d["opacity"].toLocaleString()}
                        `)
                    .style("left", `${event.pageX + 10}px`)
                    .style("top", `${event.pageY - 20}px`);
            })
            .on("mouseout", () => {
                tooltip.style("visibility", "hidden");
            });

        const allStates = new Set(schengenData.map(d => d["Schengen State"]));
        addLegend(svg, allStates, colorScale)
    }

    const opacitySelector = document.getElementById("visa-type-selector-bubble-opacity")
    opacitySelector.addEventListener("change", (event) => {
        opacityDim = event.target.value;
        updateBubble();
    });

    const sizeSelector = document.getElementById("visa-type-selector-bubble-size")
    sizeSelector.addEventListener("change", (event) => {
        sizeDim = event.target.value;
        updateBubble();
    });

    const dims = getDimensions(schengenData1)
    opacityDim = dims[0]
    sizeDim = dims[0]
    updateBubble()
});
