import {addSvg, width, height as _height, margin, getDimensions, loadData} from "./lib.js";

const height = _height-120

const svg = addSvg("#parallel-coordinates-container", 0, 60)

const loadingIndicator = d3.select("#loading");

function showLoading() {
    loadingIndicator.style("display", "block");
}

function hideLoading() {
    loadingIndicator.style("display", "none");
}

showLoading();
loadData().then(([data,_]) => {
    data.forEach(d => {
        for (const key in d) {
            if (!isNaN(+d[key])) {
                d[key] = +d[key];
            }
        }
    });

    const dimensions = getDimensions(data)

    const yScales = {};
    const epsilon = 0.1; // Small positive constant to avoid log(0)
    dimensions.forEach(dim => {
        const extent = d3.extent(data, d => d[dim]);
        const adjustedExtent = [extent[0] + epsilon, extent[1] + epsilon];
        yScales[dim] = d3.scaleLog()
            .domain(adjustedExtent)
            .range([height - margin.top - margin.bottom - 30, 0]);
    });


    const xScale = d3.scalePoint()
        .domain(dimensions)
        .range([0, width-margin.left-margin.right]);

    const lineGenerator = d3.line()
        .defined(d => !isNaN(d[1]))
        .x(d => xScale(d[0]))
        .y(d => yScales[d[0]](d[1] + epsilon));

    const groupedData = d3.group(data, d => d["Schengen State"]);
    const states = Array.from(groupedData.keys());

    const dropdown = d3.select("#state-dropdown");
    dropdown.selectAll("option")
        .data(states)
        .enter()
        .append("option")
        .attr("value", d => d)
        .text(d => d);

    function updatePlot(state) {
        const filteredData = state ? groupedData.get(state) : data;

        const paths = svg.selectAll(".data-line")
            .data(filteredData);

        paths.enter()
            .append("path")
            .attr("class", "data-line")
            .merge(paths)
            .attr("d", d => lineGenerator(dimensions.map(dim => [dim, d[dim]])))
            .style("stroke", "steelblue")
            .style("stroke-width", 2)
            .style("opacity", 0.05)
            .attr("fill", "none");

        paths.exit().remove();
    }

    // Initialize with full dataset
    updatePlot(null);

    dimensions.forEach(dim => {
        svg.append("g")
            .attr("transform", `translate(${xScale(dim)},0)`)
            .call(d3.axisLeft(yScales[dim]));

        svg.append("foreignObject")
            .attr("x", xScale(dim) - 30)
            .attr("y", height - margin.top - margin.bottom-20)
            .attr("width", 60)
            .attr("height", 140)
            .append("xhtml:div")
            .style("font-size", "12px")
            .style("text-align", "center")
            .style("word-wrap", "break-word")
            .text(dim);
    });


    dropdown.on("change", function() {
        const selectedState = d3.select(this).property("value");
        updatePlot(selectedState);
    });

    hideLoading();
}).catch(err => {
    console.error("Error loading data: ", err);
    hideLoading();
});