import {addSvg, width, height, loadData, navigate} from "./lib.js";


let currentData;
let hierarchyStack = ["Schengen States"];
let treeHistory = [];

loadData().then(([data,_]) => {
    currentData = data;
    updateTree("Schengen States");
});

function insertHistory(history, tree) {
    for (let i = history.length-1; i >= 0; i--) {
        tree = {name: history[i], children: [tree]}
    }
    return tree
}

function updateTree(filterKey) {
    // Filter data based on the selected key
    let filteredData;

    if (filterKey === "Schengen States") {
        filteredData = d3.group(
            currentData,
            d => d["Schengen State"],
            d => d["Country where consulate is located"],
            d => d["Consulate"]
        );
    } else if (hierarchyStack.length === 2) {
        // Filter for countries in the selected state
        filteredData = d3.group(
            currentData.filter(d => d["Schengen State"] === filterKey),
            d => d["Country where consulate is located"],
            d => d["Consulate"]
        );
    } else if (hierarchyStack.length === 3) {
        // Filter for consulates in the selected country
        const selectedCountry = hierarchyStack[2];
        filteredData = d3.group(
            currentData.filter(d => d["Country where consulate is located"] === selectedCountry),
            d => d["Consulate"]
        );
    }

    // Build hierarchy
    const rootData = buildHierarchy(filteredData, filterKey);
    const treeData = insertHistory(treeHistory, rootData);
    const root = d3.hierarchy(treeData);

    // Funktion, um Skalierungen für finale Knoten pro Ebene zu definieren
    function createSizeScale(nodes) {
        const values = nodes
            .filter(d => !d.children) // Nur Endknoten
            .map(d => {
                const match = d.data.name.match(/: (\d+)$/); // Extrahiere Wert aus Name
                return match ? +match[1] : 0;
            });
        return d3.scaleSqrt()
            .domain([0, d3.max(values)])
            .range([5, 40]);
    }

    const sizeScale = createSizeScale(root.descendants());

    function removeChildren(child) {
        if (child.children) {
            child._children = child.children;
            child.children = null;
        }
    }

    //Remove children that are not direct successors of current node
    if (root.children) {
        switch (treeHistory.length) {
            case 0:root.children.forEach(child=>removeChildren(child));break;
            case 1:root.children[0].children.forEach(child=>removeChildren(child));break;
            case 2:root.children[0].children[0].children.forEach(child=>removeChildren(child));break;
            case 3:root.children[0].children[0].children[0].children.forEach(child=>removeChildren(child));break;
        }
    }

    // Tree layout
    const treeLayout = d3.tree().size([height-10, width-200]);
    treeLayout(root);

    // Clear previous visualization
    d3.select("#tree-container svg").remove();

    const svg = addSvg("#tree-container")

    // Links
    svg.selectAll(".link")
        .data(root.links())
        .enter()
        .append("path")
        .attr("class", "link")
        .attr("d", d3.linkHorizontal()
            .x(d => d.y)
            .y(d => d.x)
        );

    // Nodes
    const node = svg.selectAll(".node")
        .data(root.descendants())
        .enter()
        .append("g")
        .attr("class", "node")
        .attr("transform", d => `translate(${d.y},${d.x})`);

    node.append("circle")
        .style("cursor", hierarchyStack.length < 3 ? "pointer" : "")
        .attr("r", d => {
            if (!d.children) { // Nur für finale Knoten die Größe anpassen
                const match = d.data.name.match(/: (\d+)$/);
                return match ? sizeScale(+match[1]) : 5; // Skalierung anwenden
            }
            return 5; // Standardgröße für interne Knoten
        })
        .on("click", (event, d) => {
            if (hierarchyStack.length < 3 && d.depth === 1 || d.depth === 2) { // Go deeper for states or countries
                d3.select(".tooltip").remove();
                treeHistory.push(hierarchyStack[hierarchyStack.length-1]);
                hierarchyStack.push(d.data.name);
                updateTree(d.data.name);
                updateBackButton();
                d3.selectAll(".tooltip").remove();
            }
        })
        .on("mouseover", (event, d) => {
            const tooltip = d3.select("body").append("div")
                .attr("class", "tooltip")
                .style("left", `${event.pageX + 5}px`)
                .style("top", `${event.pageY - 5}px`)
                .text(d.data.name);
        })
        .on("mouseout", () => {
            d3.selectAll(".tooltip").remove();
        });

    node.append("text")
        .attr("dy", 3)
        .attr("x", d => {
            if (d.data.name==="Schengen States") return 46
            if (d.children) return -10
            if (!d.children) { // Nur für finale Knoten die Größe anpassen
                const match = d.data.name.match(/: (\d+)$/);
                return match ? sizeScale(+match[1])+10 : 10; // Skalierung anwenden
            }
        })
        .attr("y", d => {
            if (d.data.name==="Schengen States") return -12
            if (d.children) return -7
            if (!d.children) return 0
        })
        .style("text-anchor", d => d.children ? "end" : "start")
        .text(d => d.data.name)

    d3.selectAll(".link-to-parallel").remove()
    d3.selectAll(".node").filter((d, i) => i < 2).each(function(d) {
        if (d.children&&d.data.name!=="Schengen States"){
            d3.select(this)
                .append('foreignObject')
                .attr('width', 250)
                .attr('height', 50)
                .attr('transform', 'translate(-125, 0)')
                .append('xhtml:button')
                .attr("class", "button button-inner button-regular link-to-parallel")
                .text('View as Parallel Coordinates')
                .style('cursor', 'pointer')
                .on('click', function() {
                    const stateSelector = d3.select('#state-dropdown');
                    stateSelector.property('value', d.data.name);
                    stateSelector.node().dispatchEvent(new Event('change'));
                    navigate("parallel")
                });
        }
    });


}


function buildHierarchy(nest, name) {
    return {
        name,
        children: Array.from(nest, ([key, value]) => {
            if (!key) return null;
            return value instanceof Map
                ? buildHierarchy(value, key)
                : { name: `${key}: ${d3.sum(value, d => +d["Total  uniform visas issued (including MEV)"] || 0)}` };
        }).filter(d => d)
    };
}

function updateBackButton() {
    const backButton = document.getElementById("back-button");
    backButton.disabled = hierarchyStack.length <= 1;
}

document.getElementById("back-button").addEventListener("click", () => {
    if (hierarchyStack.length > 1) {
        hierarchyStack.pop();
        treeHistory.pop();
        const previousKey = hierarchyStack[hierarchyStack.length - 1];
        updateTree(previousKey);
        updateBackButton();
    }
});